#include "mpi.h"
#include <stdio.h>
#include <math.h>

double f( double );
double f( double a )
{
    return (4.0 / (1.0 + a*a));
}

int main( int argc, char *argv[])
{
   int n_intervals_default = 2147483647;
   int n_intervals;

   int n_pass_default = 1;
   int n_pass, i_pass;

   int done = 0, n, myid, numprocs, i;
   double PI25DT = 3.141592653589793238462643;
   double mypi, pi, h, sum, x;
   double startwtime = 0.0, endwtime;
   int  namelen;
   char processor_name[MPI_MAX_PROCESSOR_NAME];

   MPI_Init(&argc,&argv);
   MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
   MPI_Comm_rank(MPI_COMM_WORLD,&myid);
   MPI_Get_processor_name(processor_name,&namelen);

   fprintf(stderr,"Process %d on %s\n",myid, processor_name);
   if( myid == 0 ) {
		if( argc > 1 ) 
			sscanf(argv[1],"%d",&n_intervals);
		else 
			n_intervals=n_intervals_default;

		if( argc > 2 ) 
			sscanf(argv[2],"%d",&n_pass);
		else 
			n_pass=n_pass_default;

		fprintf(stderr,"n_intervals=%d\n",n_intervals);
		fprintf(stderr,"n_pass=%d\n",n_pass);
	}

   n = 0;
   while (!done)
   {
      if (myid == 0) {
         startwtime = MPI_Wtime(); 
      }
	   if( n == 0  ) n = n_intervals; else n = 0;
      MPI_Bcast(&n, 1, MPI_INT, 0, MPI_COMM_WORLD);
      if (n == 0)
         done = 1;
      else
      {
			for( i_pass = 0; i_pass < n_pass; i_pass++ ) {
				h   = 1.0 / (double) n;
				sum = 0.0;
				for (i = myid + 1; i <= n; i += numprocs)
				{
					x = h * ((double)i - 0.5);
					sum += f(x);
				}
				mypi = h * sum;

				MPI_Reduce(&mypi, &pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
			}

         if (myid == 0)
         {
            printf("pi is approximately %.16f, Error is %.16f\n",
                   pi, fabs(pi - PI25DT));
            endwtime = MPI_Wtime();
            printf("wall clock time = %f\n",
            endwtime-startwtime);          
         } 
      }
   }
   MPI_Finalize();

   return 0;
}
